import {
  Metadata, IDictionary, IEventParam, Http, Router, Loader,
} from '@zeedhi/core';
import {
  Text, Row, IComponentRender, Col, Button, Textarea, AlertService,
} from '@zeedhi/common';
import {
  IterableComponentRender,
} from '@zeedhi/teknisa-components-common';
import { Client } from '@pusher/push-notifications-web';
import {
  getRouteParams,
} from '../router';

export class MinhasTarefasController {
  public hierarquiasController = Loader.getInstance('HierarquiasController');

  // Controla a visibilidade dos elementos, referente ao nível de acesso do usuário
  public isDisabledElement = (localStorage.getItem('idpermissstarefas') === 'V');

  public list: IDictionary = [];

  public listCardTarefa: IDictionary = [];

  public async createCardNextTask() {
    const hierarchyUsersList = await this.hierarquiasController.mountHierarchyUsersArray(); // Busca os usuários subordinados do usuário logado

    Http.post('/getNextTask', { usersIds: hierarchyUsersList }).then((response: IDictionary) => {
      const firstTask = response.data.dataset.nextTC.filter((value: IDictionary) => value.IDSITUACAOAGEND === 'Em andamento');
      if (firstTask.length) {
        this.responseFirstTask(firstTask);
        this.changeTitleAndButton();
        const index = response.data.dataset.nextTC.findIndex((value: IDictionary) => value.IDSITUACAOAGEND === 'Em andamento');
        response.data.dataset.nextTC.splice(index, 1);
        this.responseAddIterable(response.data.dataset.nextTC);
      } else {
        this.responseFirstTask(response.data.dataset.nextTC.slice(0, 1));
        this.responseAddIterable(response.data.dataset.nextTC.slice(1));
      }
    });
  }

  public responseAddIterable(response: IDictionary) {
    Metadata.getInstance<IterableComponentRender>('iCardsTaskScheduled').datasource.data = [];
    response.forEach((dados: IDictionary) => {
      const DateTime = dados.DTHORAAGEND.split(' ');
      Metadata.getInstance<IterableComponentRender>('iCardsTaskScheduled').datasource.data.push({
        NRTAREFA: dados.NRCHECKLISTAGEND,
        DTHORA: DateTime[1],
        DATAH: DateTime[0],
        IDSITUACAO: dados.IDSITUACAOAGEND,
        NMTAREFA: dados.NMCHECKLISTAGEND,
        NMLOCAL: dados.NMLOCALCLIENTE,
        NMTIPOLOCAL: dados.NMLOCALPAI,
      });
    });
  }

  public responseFirstTask(response: IDictionary) {
    response.forEach((dados: IDictionary) => {
      const DateTime = dados.DTHORAAGEND.split(' ');
      Metadata.getInstance<Button>('buttonReport').to = `/Reportar_Tarefa/${dados.NRCHECKLISTAGEND}`;
      Metadata.getInstance<Row>('rowCardNextTask').children = [this.modelCards({
        NRCHECKLISTAGEND: dados.NRCHECKLISTAGEND,
        HORAAGEND: DateTime[1],
        DTAGEND: DateTime[0],
        IDSITUACAOAGEND: dados.IDSITUACAOAGEND,
        NMCHECKLISTAGEND: dados.NMCHECKLISTAGEND,
        NMLOCALCLIENTE: dados.NMLOCALCLIENTE,
        NRLOCALCLIENTEPAI: dados.NMLOCALPAI,
      }) as IComponentRender];
    });
  }

  public modelCards(dataBase: IDictionary) {
    return {
      name: `colCardsMain_${dataBase.NRCHECKLISTAGEND}`,
      component: 'ZdCol',
      cssClass: 'zd-px-0 zd-py-0 zd-mt-1',
      children: [
        {
          name: 'cardsMain',
          component: 'ZdCard',
          cssClass: 'zd-mb-1',
          to: `/Detalhes_Tarefa/${dataBase.NRCHECKLISTAGEND}`,
          children: [
            {
              name: 'rowCardsMain',
              component: 'ZdRow',
              children: [
                {
                  name: 'colInternalDateTime',
                  component: 'ZdCol',
                  cssClass: 'zd-my-auto',
                  cols: 3,
                  children: [
                    {
                      name: 'rowCardTime',
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'textTime',
                          component: 'ZdText',
                          text: dataBase.HORAAGEND,
                          style: {
                            color: '#495057',
                            'font-size': '20px',
                            'font-weight': 'bold',
                          },
                        },
                      ],
                    },
                    {
                      name: 'rowCardDate',
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'textDate',
                          component: 'ZdText',
                          text: dataBase.DTAGEND,
                          style: {
                            color: '#6c757d',
                            'font-size': '14px',
                            'font-weight': 'bold',
                            'margin-left': '6px',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'colInternalDataBase',
                  component: 'ZdCol',
                  cols: 9,
                  children: [
                    {
                      name: 'rowCardStatus',
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'statusTask',
                          component: 'ZdText',
                          text: dataBase.IDSITUACAOAGEND,
                          events: {
                            onBeforeMount: '{{MinhasTarefasController.stylizeStatus}}',
                          },
                        },
                      ],
                    },
                    {
                      name: 'rowCardTask',
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'nameTask',
                          component: 'ZdText',
                          text: dataBase.NMCHECKLISTAGEND,
                          style: {
                            color: '#6c757d',
                            'font-size': '14px',
                          },
                        },
                      ],
                    },
                    {
                      name: 'rowCardLocalTask',
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'localTask',
                          component: 'ZdText',
                          text: dataBase.NMLOCALCLIENTE,
                          style: {
                            color: '#495057',
                            'font-size': '18px',
                            'font-weight': 'bold',
                          },
                        },
                      ],
                    },
                    {
                      name: 'rowTypeLocal',
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'localTask',
                          component: 'ZdText',
                          text: dataBase.NRLOCALCLIENTEPAI,
                          style: {
                            color: '#6c757d',
                            'font-size': '14px',
                          },
                        },
                      ],
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  public stylizeStatus({ component }: IEventParam<Text>) {
    if (component.text === 'Pendente') {
      component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 10px;'>P E N D E N T E</span>";
    }
    if (component.text === 'Concluído') {
      component.text = "<span  style='background-color: #dee2e6; color: green; font-size: 10px;'>C O N C L U Í D O</span>";
    }
    if (component.text === 'Em andamento') {
      component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
    }
    if (component.text === 'Cancelado') {
      component.text = "<span  style='background-color: #dee2e6; color: black; font-size: 10px;'>C A N C E L A D O</span>";
    }
    if (component.text === 'Atrasado') {
      component.text = "<span  style='background-color: #dee2e6; color: red; font-size: 10px;'>A T R A S A D O</span>";
    }
  }

  public changeTitleAndButton() {
    const title = Metadata.getInstance<Text>('titleNextTask');
    title.text = (title.text === 'Próxima Tarefa') ? 'Tarefa Aberta' : 'Próxima Tarefa';
    const buttonStart = Metadata.getInstance<Col>('colButtonStart');
    const buttonPress = Metadata.getInstance<Col>('colButtonPress');
    buttonStart.isVisible = !buttonStart.isVisible;
    buttonPress.isVisible = !buttonPress.isVisible;
  }

  public changeStatusTask({ component }:{ component:Button }) {
    Http.post('/changeStatusTask', {
      button: (component.name === 'buttonStart') ? 2 : 4,
      NRTAREFAATIVA: (Number(component!.parent!.parent!.parent!.getChildInstance<Row>('rowCardNextTask').children[0].name.split('_')[1])),
    }).then(() => {
      this.createCardNextTask();
    });
    if (component.name === 'buttonPress') {
      this.changeTitleAndButton();
    } else {
      const row: any = Metadata.getInstance<Row>('rowCardsMain')!.children[1];
      const cols = row!.children[0];
      cols!.children[0]!.text = 'EM ANDAMENTO';
      cols!.children[0]!.cssClass = 'cStatusAndamento';
    }
  }

  public report() {
    const { nr } = getRouteParams();
    let justify = ['buttonBusy', 'buttonWtAcess', 'buttonCleanPlace', 'buttonOtherReasons'].filter((nameButton: string) => Metadata.getInstance<Button>(nameButton).outline === false)[0];
    if (justify === 'buttonOtherReasons') {
      justify = (Metadata.getInstance<Textarea>('textarea').value.trim().replace(' ', '').length > 0) ? Metadata.getInstance<Textarea>('textarea').value.trim() : justify;
    } else {
      justify = Metadata.getInstance<Button>(justify).label;
    }
    if (justify !== 'buttonOtherReasons') {
      Http.post('/changeStatusTask', {
        button: 3,
        NRTAREFAATIVA: nr,
        DSJUSTCANCELAMENTO: justify,
      }).then(() => {
        Router.replace('/Tarefas');
      });
    } else {
      AlertService.show({
        name: 'basic-alert',
        text: 'Favor preencher o motivo!',
        top: true,
        left: false,
        timeout: 3000,
        color: 'error',
      });
    }
  }

  public subscribeTasksNotifications() {
    const beamsClient = new Client({
      instanceId: process.env.VUE_APP_PUSH_NOTIFICATION_INSTANCE_ID,
    });
    Notification.requestPermission()
      .then((result) => {
        if (result === 'granted') {
          beamsClient.start()
            .then(() => {
              const nrUser = window.localStorage.getItem('UserLogado');
              const devInterest = `tasks-${nrUser}`;
              beamsClient.addDeviceInterest(devInterest)
                .then(() => {
                  AlertService.show({
                    name: 'basic-alert',
                    text: 'Agora você receberá notificações sobre suas vistorias',
                    top: true,
                    left: false,
                    timeout: 3000,
                    color: 'success',
                  });
                })
                .catch((err : any) => {
                  AlertService.show({
                    name: 'basic-alert',
                    text: err,
                    top: true,
                    left: false,
                    timeout: 3000,
                    color: 'error',
                  });
                });
            })
            .catch((err : any) => {
              AlertService.show({
                name: 'basic-alert',
                text: err,
                top: true,
                left: false,
                timeout: 3000,
                color: 'error',
              });
            });
        } else {
          AlertService.show({
            name: 'basic-alert',
            text: 'Você não permitiu que as notificações sejam enviadas. Você pode alterar isso se mudar de ideia depois.',
            top: true,
            left: false,
            timeout: 3000,
            color: 'warning',
          });
        }
      })
      .catch((err) => {
        AlertService.show({
          name: 'basic-alert',
          text: err,
          top: true,
          left: false,
          timeout: 3000,
          color: 'error',
        });
      });
  }
}
